@@ -47,35 +47,41 @@ static VALUE callback_exception(VALUE unused) {
 } 
 
 static void curl_multi_mark(ruby_curl_multi *rbcm) {
-  rb_gc_mark(rbcm->requests);
+  if (!NIL_P(rbcm->requests)) rb_gc_mark(rbcm->requests);
 }
 
-static void curl_multi_flush_easy(VALUE key, VALUE easy, ruby_curl_multi *rbcm) {
+/* Hash#foreach callback for curl_multi_free */
+static int curl_multi_flush_easy(VALUE key, VALUE easy, ruby_curl_multi *rbcm) {
   CURLMcode result;
   ruby_curl_easy *rbce;
 
-  Data_Get_Struct(easy, ruby_curl_easy, rbce);
-  result = curl_multi_remove_handle(rbcm->handle, rbce->curl);
-  if (result != 0) {
-    raise_curl_multi_error_exception(result);
+  // sometimes the type is T_ZOMBIE, e.g. after Ruby has received the SIGTERM signal
+  if (rb_type(easy) == T_DATA) {
+    Data_Get_Struct(easy, ruby_curl_easy, rbce);
+
+    result = curl_multi_remove_handle(rbcm->handle, rbce->curl);
+    if (result != 0) {
+      raise_curl_multi_error_exception(result);
+    }
   }
-}
 
-static int
-rb_hash_clear_i(VALUE key, VALUE value, VALUE dummy) {
   return ST_DELETE;
 }
 
-static void curl_multi_free(ruby_curl_multi *rbcm) {
+void curl_multi_free(ruby_curl_multi *rbcm) {
+  VALUE hash = rbcm->requests;
 
-  if (rbcm && !rbcm->requests == Qnil && rb_type(rbcm->requests) == T_HASH && RHASH_SIZE(rbcm->requests) > 0) {
+  if (!NIL_P(hash) && rb_type(hash) == T_HASH && RHASH_SIZE(hash) > 0) {
 
-    rb_hash_foreach( rbcm->requests, (int (*)())curl_multi_flush_easy, (VALUE)rbcm );
+    rb_hash_foreach(hash, curl_multi_flush_easy, (VALUE)rbcm);
+    /* rb_hash_clear(rbcm->requests); */
 
-    rb_hash_foreach(rbcm->requests, rb_hash_clear_i, 0); //rb_hash_clear(rbcm->requests);
     rbcm->requests = Qnil;
   }
-  curl_multi_cleanup(rbcm->handle);
+
+  if (rbcm->handle) {
+    curl_multi_cleanup(rbcm->handle);
+  }
   free(rbcm);
 }
 
@@ -114,7 +120,7 @@ VALUE ruby_curl_multi_new(VALUE klass) {
  *
  */
 VALUE ruby_curl_multi_set_default_timeout(VALUE klass, VALUE timeout) {
-  cCurlMutiDefaulttimeout = FIX2LONG(timeout);
+  cCurlMutiDefaulttimeout = NUM2LONG(timeout);
   return timeout;
 }
 
@@ -126,7 +132,7 @@ VALUE ruby_curl_multi_set_default_timeout(VALUE klass, VALUE timeout) {
  *
  */
 VALUE ruby_curl_multi_get_default_timeout(VALUE klass) {
-  return INT2FIX(cCurlMutiDefaulttimeout);
+  return LONG2NUM(cCurlMutiDefaulttimeout);
 }
 
 /* Hash#foreach callback for ruby_curl_multi_requests */
@@ -168,7 +174,7 @@ static VALUE ruby_curl_multi_idle(VALUE self) {
   
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
   
-  if ( FIX2INT( rb_funcall(rbcm->requests, rb_intern("length"), 0) ) == 0 ) {
+  if (RHASH_SIZE(rbcm->requests) == 0) {
     return Qtrue;
   } else {
     return Qfalse;
@@ -187,7 +193,15 @@ static VALUE ruby_curl_multi_max_connects(VALUE self, VALUE count) {
   ruby_curl_multi *rbcm;
 
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
-  curl_multi_setopt(rbcm->handle, CURLMOPT_MAXCONNECTS, NUM2INT(count));
+
+  if (!rbcm->handle) {
+    rbcm->handle = curl_multi_init();
+    if (!rbcm->handle) {
+      rb_raise(mCurlErrFailedInit, "Failed to initialize multi handle");
+    }
+  }
+
+  curl_multi_setopt(rbcm->handle, CURLMOPT_MAXCONNECTS, NUM2LONG(count));
 #endif
 
   return count;
@@ -198,20 +212,39 @@ static VALUE ruby_curl_multi_max_connects(VALUE self, VALUE count) {
  * multi = Curl::Multi.new
  * multi.pipeline = true
  *
- * Pass a long set to 1 to enable or 0 to disable. Enabling pipelining on a multi handle will make it
- * attempt to perform HTTP Pipelining as far as possible for transfers using this handle. This means
- * that if you add a second request that can use an already existing connection, the second request will
- * be "piped" on the same connection rather than being executed in parallel. (Added in 7.16.0)
+ * Pass a long set to 1 for HTTP/1.1 pipelining, 2 for HTTP/2 multiplexing, or 0 to disable.
+ *  Enabling pipelining on a multi handle will make it attempt to perform HTTP Pipelining as 
+ * far as possible for transfers using this handle. This means that if you add a second request 
+ * that can use an already existing connection, the second request will be "piped" on the same
+ * connection rather than being executed in parallel. (Added in 7.16.0, multiplex added in 7.43.0)
  *
  */
-static VALUE ruby_curl_multi_pipeline(VALUE self, VALUE onoff) {
+static VALUE ruby_curl_multi_pipeline(VALUE self, VALUE method) {
 #ifdef HAVE_CURLMOPT_PIPELINING
   ruby_curl_multi *rbcm;
 
+  long value;
+
+  if (method == Qtrue) {
+    value = 1;
+  } else if (method == Qfalse) {
+    value  = 0;
+  } else {
+    value = NUM2LONG(method);
+  } 
+
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
-  curl_multi_setopt(rbcm->handle, CURLMOPT_PIPELINING, onoff == Qtrue ? 1 : 0);
+
+  if (!rbcm->handle) {
+    rbcm->handle = curl_multi_init();
+    if (!rbcm->handle) {
+      rb_raise(mCurlErrFailedInit, "Failed to initialize multi handle");
+    }
+  }
+
+  curl_multi_setopt(rbcm->handle, CURLMOPT_PIPELINING, value);
 #endif
-  return onoff;
+  return method == Qtrue ? 1 : 0;
 }
 
 /*
@@ -225,15 +258,29 @@ static VALUE ruby_curl_multi_pipeline(VALUE self, VALUE onoff) {
  */
 VALUE ruby_curl_multi_add(VALUE self, VALUE easy) {
   CURLMcode mcode;
+  VALUE r;
   ruby_curl_easy *rbce;
   ruby_curl_multi *rbcm;
 
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
   Data_Get_Struct(easy, ruby_curl_easy, rbce);
 
+  // check if this curl handle has been added before adding again
+  r = rb_hash_aref(rbcm->requests, LONG2NUM((long)rbce->curl));
+  if ( r != Qnil ) {
+    return Qnil;
+  }
+
   /* setup the easy handle */
   ruby_curl_easy_setup( rbce );
 
+  if (!rbcm->handle) {
+    rbcm->handle = curl_multi_init();
+    if (!rbcm->handle) {
+      rb_raise(mCurlErrFailedInit, "Failed to initialize multi handle");
+    }
+  }
+
   mcode = curl_multi_add_handle(rbcm->handle, rbce->curl);
   if (mcode != CURLM_CALL_MULTI_PERFORM && mcode != CURLM_OK) {
     raise_curl_multi_error_exception(mcode);
@@ -245,7 +292,10 @@ VALUE ruby_curl_multi_add(VALUE self, VALUE easy) {
    * If this number is not correct, the next call to curl_multi_perform will correct it. */
   rbcm->running++;
 
-  rb_hash_aset( rbcm->requests, easy, easy );
+  /* track a reference to associated multi handle */
+  rbce->multi = self;
+
+  rb_hash_aset( rbcm->requests, LONG2NUM((long)rbce->curl), easy );
 
   return self;
 }
@@ -269,6 +319,13 @@ VALUE ruby_curl_multi_remove(VALUE self, VALUE easy) {
 
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
 
+  if (!rbcm->handle) {
+    rbcm->handle = curl_multi_init();
+    if (!rbcm->handle) {
+      rb_raise(mCurlErrFailedInit, "Failed to initialize multi handle");
+    }
+  }
+
   rb_curl_multi_remove(rbcm,easy);
 
   return self;
@@ -280,6 +337,12 @@ static void rb_curl_multi_remove(ruby_curl_multi *rbcm, VALUE easy) {
 
   Data_Get_Struct(easy, ruby_curl_easy, rbce);
 
+  // check if this curl handle has been added before removing
+  r = rb_hash_aref(rbcm->requests, LONG2NUM((long)rbce->curl));
+  if ( r == Qnil ) {
+    return;
+  }
+
   result = curl_multi_remove_handle(rbcm->handle, rbce->curl);
   if (result != 0) {
     raise_curl_multi_error_exception(result);
@@ -289,8 +352,8 @@ static void rb_curl_multi_remove(ruby_curl_multi *rbcm, VALUE easy) {
 
   ruby_curl_easy_cleanup( easy, rbce );
 
-  // active should equal INT2FIX(RHASH(rbcm->requests)->tbl->num_entries)
-  r = rb_hash_delete( rbcm->requests, easy );
+  // active should equal LONG2NUM(RHASH(rbcm->requests)->tbl->num_entries)
+  r = rb_hash_delete( rbcm->requests, LONG2NUM((long)rbce->curl) );
   if( r != easy || r == Qnil ) {
     rb_warn("Possibly lost track of Curl::Easy VALUE, it may not be reclaimed by GC");
   }
@@ -313,6 +376,13 @@ static VALUE ruby_curl_multi_cancel(VALUE self) {
   ruby_curl_multi *rbcm;
 
   Data_Get_Struct(self, ruby_curl_multi, rbcm);
+
+  if (!rbcm->handle) {
+    rbcm->handle = curl_multi_init();
+    if (!rbcm->handle) {
+      rb_raise(mCurlErrFailedInit, "Failed to initialize multi handle");
+    }
+  }
   
   rb_hash_foreach( rbcm->requests, ruby_curl_multi_cancel_callback, (VALUE)rbcm );
   
@@ -406,8 +476,6 @@ static void rb_curl_mutli_handle_complete(VALUE self, CURL *easy_handle, int res
 
   if (val == Qfalse) {
     rb_warn("uncaught exception from callback");
-      // exception was raised?
-    //fprintf(stderr, "exception raised from callback\n");
   }
 
 }
@@ -437,10 +505,11 @@ static void rb_curl_multi_run(VALUE self, CURLM *multi_handle, int *still_runnin
     mcode = curl_multi_perform(multi_handle, still_running);
   } while (mcode == CURLM_CALL_MULTI_PERFORM);
 
+
   if (mcode != CURLM_OK) {
     raise_curl_multi_error_exception(mcode);
   }
-  
+
 }
 
 #ifdef _WIN32
@@ -504,7 +573,7 @@ static VALUE curb_select(void *args) {
 VALUE ruby_curl_multi_perform(int argc, VALUE *argv, VALUE self) {
   CURLMcode mcode;
   ruby_curl_multi *rbcm;
-  int maxfd, rc;
+  int maxfd, rc = -1;
   fd_set fdread, fdwrite, fdexcep;
 #ifdef _WIN32
   fd_set crt_fdread, crt_fdwrite, crt_fdexcep;
@@ -614,7 +683,15 @@ VALUE ruby_curl_multi_perform(int argc, VALUE *argv, VALUE self) {
 
   rb_curl_multi_read_info( self, rbcm->handle );
   if (block != Qnil) { rb_funcall(block, rb_intern("call"), 1, self);  }
-    
+
+  /* do early cleanup */
+  VALUE hash = rbcm->requests;
+  if (!NIL_P(hash) && rb_type(hash) == T_HASH && RHASH_SIZE(hash) > 0) {
+    rb_hash_foreach(hash, curl_multi_flush_easy, (VALUE)rbcm);
+  }
+  curl_multi_cleanup(rbcm->handle);
+  rbcm->handle = NULL;
+ 
   return Qtrue;
 }
 
@@ -633,7 +710,7 @@ void init_curb_multi() {
   rb_define_method(cCurlMulti, "requests", ruby_curl_multi_requests, 0);
   rb_define_method(cCurlMulti, "idle?", ruby_curl_multi_idle, 0);
   
-  /* Instnace methods */
+  /* Instance methods */
   rb_define_method(cCurlMulti, "max_connects=", ruby_curl_multi_max_connects, 1);
   rb_define_method(cCurlMulti, "pipeline=", ruby_curl_multi_pipeline, 1);
   rb_define_method(cCurlMulti, "add", ruby_curl_multi_add, 1);
